library(gplots)
library(RColorBrewer)

bcm_root <- Sys.getenv("BCM_ROOT")
source(paste(bcm_root, "scripts/plots_functions.r", sep="/"))

model_name <- "shells10D"
output_folder <- "output_pt"

model <- load_model(model_name, output_folder)

# Density overview
print("Plotting marginal densities...")
outputfn <- paste(model$output_folder, "/marginal_densities.png", sep="");
png(outputfn, width=1800, height=1000);
par(mfrow=c(ceiling(sqrt(model$nvar)),ceiling(sqrt(model$nvar))))
par(mar=c(2,2,3,2))
for (i in 1:model$nvar) {
  varattrs <- xmlAttrs(model$vars[[i]])
  plot_variable_distribution(model$samples[,i], varattrs)
}
par(mfrow=c(1,1))
res <- dev.off();

# Autocorrelation plots
print("Plotting ACF...")
outputfn <- paste(model$output_folder, "/acfs.png", sep="");
png(outputfn, width=1800, height=1000);
par(mfrow=c(ceiling(sqrt(model$nvar)),ceiling(sqrt(model$nvar))))
par(mar=c(2,2,3,2))
for (i in 1:model$nvar) {
  varattrs <- xmlAttrs(model$vars[[i]])
  acf(model$samples[,i], main=varattrs["name"])
}
par(mfrow=c(1,1))
res <- dev.off();

# Traces
print("Plotting traces...")
outputfn <- paste(model$output_folder, "/traces.png", sep="");
png(outputfn, width=1800, height=1000);
par(mfrow=c(ceiling(sqrt(model$nvar)),ceiling(sqrt(model$nvar))))
par(mar=c(2,2,3,2))
for (i in 1:model$nvar) {
  varattrs <- xmlAttrs(model$vars[[i]])
  plot(model$samples[,i], main=varattrs["name"], xlab="Sample index", ylab="Value")
}
par(mfrow=c(1,1))
res <- dev.off();

# Convergence overview
outputfn <- paste(model$output_folder, "/convergence.png", sep="");
png(outputfn, width=1800, height=1000);
par(mfcol=c(2,2))
plot(model$samples[,1], main="First variable trace", xlab="Sample index", ylab="Value")
acf(model$samples[,1], lag.max=nrow(model$samples)/10, main="First variable ACF")
plot(model$samples[,model$nvar+3], main="Posterior probability", xlab="Sample index", ylab="log probability")
plot(log10(model$samples[,model$nvar+4]), main="Sample weights", xlab="Sample index", ylab="log10 weight")
res <- dev.off();

# Correlation & convergence
print("Plotting correlation...")
samples <- model$samples[,1:model$nvar]
co <- cor(samples)
rownames(co) <- 1:model$nvar
for (i in 1:model$nvar) {
  varattrs <- xmlAttrs(model$vars[[i]])
  rownames(co)[i] <- paste(i, "-", varattrs["name"], "-", i, sep="")
}
colnames(co) <- rownames(co)

outputfn <- paste(model$output_folder, "/parameter_correlation.png", sep="");
png(outputfn, width=3000, height=3000);
heatmap.2(1-abs(co), trace="none", col=brewer.pal(9, "Reds"), Rowv=F, Colv=F, dendrogram="none", margins=c(30,30), cexRow=1.5, cexCol=1.5)
res <- dev.off();

outputfn <- paste(model$output_folder, "/parameter_correlation_clustered.png", sep="");
png(outputfn, width=3000, height=3000);
heatmap.2(1-abs(co), trace="none", col=brewer.pal(9, "Reds"), margins=c(30,30), cexRow=1.5, cexCol=1.5)
res <- dev.off();
