#pragma once

namespace ModelParser {

class Variable
{
public:
	enum EType {
		T_Boolean,
		T_Real,
		T_Invalid,
	};

	Variable();
	~Variable();

	bool Parse(const std::string& line, unsigned int linenr);
	bool ParsePrior(const std::string& line, unsigned int linenr, const std::string& prior, std::string index);

	inline const std::string& GetName() const { return Name; }
	inline size_t GetNumReplicates() const { return Replicates; }
	bool GetReplicateIndex(const std::string& replicate_name, size_t& ix) const;
	inline const std::string& GetReplicateName(size_t ix) const { return ReplicateNames[ix]; }
	EType GetType() const { return Type; }

	inline bool IsLatent() const { return Latent; }
	std::string GetVariableFileString(size_t ix) const;

	void SetLatentIndex(size_t start);
	size_t GetLatentIndex() const { return LatentIndex; }

private:
	std::string Name;
	EType Type;
	size_t Replicates;
	std::vector<std::string> ReplicateNames;

	bool Latent;
	struct Prior {
		std::string PriorType;
		std::map<std::string, std::string> PriorParams;
	};
	std::vector<Prior> Priors;

	size_t LatentIndex;
};

}
