#include "Utils.h"
#include "Prior.h"
#include "NestedSamplingStrategyPrior.h"

namespace bcm {

NestedSamplingStrategyPrior::NestedSamplingStrategyPrior(SamplerNested* sampler)
	: NestedSamplingStrategy(sampler)
{
}

NestedSamplingStrategyPrior::~NestedSamplingStrategyPrior()
{
}

bool NestedSamplingStrategyPrior::GetNewSample(Real min_llh, Sample* sample, RNG* rng, size_t threadix) const
{
	// Draw sample from prior
	if (!prior->Sample(sample->variable_values.data(), rng)) {
		LOGERROR("Sampling from prior failed");
		return false;
	}

	if (!EvaluatePriorLikelihood(threadix, sample->variable_values.data(), sample->lprior, sample->llh)) {
		return false;
	}
		
	// Make sure the prior density is non-zero; in case the random sampling from the prior is inaccurate.
	if (sample->lprior == -std::numeric_limits<Real>::infinity()) {
		sample->lposterior = sample->lprior;
	} else {
		if (!GetOutputValues(threadix, sample->output_values.data())) {
			return false;
		}
		sample->lposterior = sample->lprior + sample->llh;
	}

	return true;
}

}
