#pragma once

namespace bcm {

class UnivariateMarginal
{
public:
	UnivariateMarginal();
	~UnivariateMarginal();

	bool Initialize(boost::property_tree::ptree pt);
	static UnivariateMarginal* CreateUniform(Real lower, Real upper);
	static UnivariateMarginal* CreateNormal(Real mu, Real sigma);
	static UnivariateMarginal* CreateExponential(Real lambda);
	static UnivariateMarginal* CreateGamma(Real k, Real theta);
	static UnivariateMarginal* CreateBeta(Real a, Real b);
	static UnivariateMarginal* CreateHalfCauchy(Real scale);

	bool Sample(Real& value, RNG* rng) const;
	bool EvaluatePDF(Real& p, const Real value) const;
	bool EvaluateLogPDF(Real& logp, const Real value) const;
	bool EvaluateVariance(Real& var) const;
	bool EvaluateCDF(Real& p, const Real value) const;
	bool EvaluateQuantile(Real& value, const Real p) const;
	Real GetLowerBound() const;
	Real GetUpperBound() const;

private:
	enum EDistribution
	{
		D_Invalid,
		D_Uniform,
		D_Normal,
		D_Exponential,
		D_Gamma,
		D_Beta,
		D_HalfCauchy
	};
	
	EDistribution PriorType;

	Real mu;			// Normal
	Real sigma;			// Normal
	Real a;				// Uniform, Beta
	Real b;				// Uniform, Beta
	Real lambda;		// Exponential
	Real k;				// Gamma
	Real theta;			// Gamma
	Real scale;			// HalfCauchy

	// Optimization
	Real OneOverTwoSigmaSq;
	Real LogOneOverSqrtTwoPiSigmaSq;
};

}
