#include "Utils.h"
#include "VariableSet.h"

#include <boost/property_tree/xml_parser.hpp>

namespace bcm {

VariableSet::VariableSet()
{
}

VariableSet::~VariableSet()
{
}

bool VariableSet::LoadFromXML(const std::string& filename)
{
	boost::property_tree::ptree pt;
	try {
		boost::property_tree::read_xml(filename, pt);
	} catch (boost::property_tree::xml_parser_error &e) {
		LOGERROR("Error loading variable file: %s", e.what());
		return false;
	}
	
	try {
		boost::property_tree::ptree model_node = pt.get_child("variableset");
		
		BOOST_FOREACH(const boost::property_tree::ptree::value_type& var, model_node.get_child("")) {
			if (var.first == "variable") {
				std::string name = var.second.get<std::string>("<xmlattr>.name");
				Variables.push_back(name);

				bool logspace = var.second.get<bool>("<xmlattr>.logspace", false);
				IsLogSpace.push_back(logspace);
			}
		}

		return true;
	} catch (boost::property_tree::ptree_error &e) {
		LOGERROR("Error parsing variable file: %s", e.what());
		return false;
	}
}

void VariableSet::AddVariable(const std::string& name, bool logspace)
{
	Variables.push_back(name);
	IsLogSpace.push_back(logspace);
}

size_t VariableSet::GetVariableIndex(const std::string& name) const
{
	for (size_t vi = 0; vi < Variables.size(); vi++) {
		if (Variables[vi] == name) {
			return vi;
		}
	}
	LOGERROR("Could not find variable \"%s\"", name.c_str());
	return std::numeric_limits<size_t>::max();
}

}
